import tensorflow as tf
from tensorflow import feature_column as fc

from ..utils import create_boundaries, create_vocabulary_list


def build_features(df, int_columns, float_columns, categorical_columns, cross_columns=[]):
    """
    df:
    int_columns:
    float_columns:
    categorical_columns:
    cross_columns=[]:
    return:
    """
    numeric_features = []
    for col in (int_columns + float_columns):
        if col in int_columns:
            numeric_features.append(
                fc.bucketized_column(fc.numeric_column(col, dtype=tf.int64), boundaries=create_boundaries(df, col)))
        else:
            numeric_features.append(fc.bucketized_column(fc.numeric_column(col), boundaries=create_boundaries(df, col)))

    categorical_features = []
    for col in categorical_columns:
        if col == "card_id":
            categorical_features.append(
                fc.embedding_column(fc.categorical_column_with_hash_bucket(col, 20000, dtype=tf.int64),
                                    dimension=int(df[col].size**0.25)))
        elif col == "device_id":
            categorical_features.append(
                fc.embedding_column(fc.categorical_column_with_hash_bucket(col, 400000), dimension=int(df[col].size**0.25)))
        elif col in [
                "show_tag_id", "click_tractate_id1", "click_tractate_id2", "click_tractate_id3", "click_tractate_id4",
                "click_tractate_id5", "click_diary_id1", "click_diary_id2", "click_diary_id3", "click_diary_id4",
                "click_diary_id5", "service_id", "recommend_service_id"
        ]:
            categorical_features.append(
                fc.embedding_column(fc.categorical_column_with_hash_bucket(col, 20000), dimension=int(df[col].size**0.25)))
        else:
            categorical_features.append(
                fc.indicator_column(fc.categorical_column_with_vocabulary_list(col, create_vocabulary_list(df, col))))

    for lst in cross_columns:
        column_a = lst[0]
        column_b = lst[1]
        tmp_a = fc.categorical_column_with_vocabulary_list(column_a, create_vocabulary_list(df, column_a))
        tmp_b = fc.categorical_column_with_vocabulary_list(column_b, create_vocabulary_list(df, column_b))
        tmp_res = tf.feature_column.crossed_column([tmp_a, tmp_b], hash_bucket_size=10)
        categorical_features.append(tf.feature_column.indicator_column(tmp_res))

    all_features = (numeric_features + categorical_features)
    return all_features
